// FileFilter 0.70 by Earl Colby Pottinger. 
// Partly based on code found in the Beos CDROM 
// Partly based on instructions found in "Programming the Be Operating System" By Dan Parks Sydow. 
// Additional help by Larry Baydak in debugging and coding/commenting style. 

// #include "FileFilter_Includes.h"
// #include "FileFilter_Menus.h"

// FileFilter Includes.h 0.70 by Earl Colby Pottinger. 
// Needed for basic applications.
#include <Application.h>  // Used for all programs.
#include <InterfaceKit.h>
#include <Window.h>       // Used to create windows on the desktop.
#include <View.h>         // Used to create views inside a window.
#include <Alert.h>        // Used to open BAlert Windows.

// Needed to exchange messages.
#include <Messenger.h>
#include <Message.h>
#include <Roster.h>

// Needed to create and use menus.
#include <Menu.h>
#include <MenuBar.h>
#include <MenuItem.h>

// Needed to create and use Strings & BStrings.
#include <string.h> // Posix string includes.
#include <String.h> // BString includes.
#include <StringView.h>

// Needed for input/output to devices.
#include <stdio.h>  // Basic input/output.
#include <stdlib.h> // Standard library.

// Needed to load, display, and save text files.
#include <StorageKit.h>
#include <Entry.h>
#include <Path.h>
#include <FilePanel.h>
#include <fcntl.h>
#include <unistd.h>

// FileFilter Menus.h 0.70 by Earl Colby Pottinger. 
// Constants Messages for menu commands.
const uint32 MENU_FILE_FILTER = 'Fltr'; // Filter out control codes
const uint32 MENU_FILE_ABOUT  = 'What'; // About the program.
const uint32 MENU_FILE_QUIT   = 'Quit'; // Exit program.
const uint32 MENU_FILETYPE_0  = 'FLT0'; // No FileType.
const uint32 MENU_FILETYPE_1  = 'FLT1'; // Basic Text FileType.
const uint32 MENU_FILETYPE_2  = 'FLT2'; // Plain Text FileType.
const uint32 MENU_FILETYPE_3  = 'FLT3'; // HTML Text FileType.
const uint32 MENU_FILETYPE_4  = 'FLT4'; // RTF Text FileType.
const uint32 MENU_FILETYPE_5  = 'FLT5'; // E-mail FileType.
const uint32 MENU_FILETYPE_6  = 'FLT6'; // Source Code FileType.
const uint32 MENU_FILETYPE_7  = 'FLT7'; // UseNet Message FileType.
const uint32 MENU_FILETYPE_8  = 'FLT8'; // Message RFC822 FileType.

//Constants needed. 
const char *APP_SIGNATURE="application/x-gui.FileFilter_0.70"; // Application's signature.
rgb_color blackColor = {  0,  0,  0,255};

// Objects needed.
 BFilePanel *OpenFilePanel = NULL; // Define object to create open panels from.
 BAlert     *alert;         // Create a pointer (*alert) to an alert window

 BFile   File_Object;  // BFile object.
 BPath   File_B_Path;  // Path to file (includes file name in full path).
 BEntry  File_B_Entry; // BEntry converted from entry_ref.
 BString File_Name = "";

 entry_ref file_entry_ref; // File Entry.
 off_t     File_Size;      // Length of file.
 uchar*    File_Buffer;    // Memory Buffer to store file to filter.
 status_t  File_Error;     // The error/return code
 size_t    Bytes_Read;     // Number of bytes read in from file.

// Variables needed.
 char   name[B_FILE_NAME_LENGTH];
 int    refs;          // Index into list of files passed to Addon
 int    ASC;           // Printable Ascii character,
 int    Index_Counter; // General Counter.
 long   result=0;      // Returned value from the BAlert::Go() member function.
 int    FileType=0;    // Filetype of filtered file.
 int    n;
 float X;
 float Y;

// FileFilter_View.h
class FileFilter_View : public BView {                   // Define the class object FileFilter_View.
 public:       FileFilter_View(BRect frame, char *name); // Constructor for a view inside a window.
  virtual void AttachedToWindow();                       // Called by the OS when 'aWindow->AddChild(aView)' used.
  virtual void Draw(BRect updateRect); } ;               // Called by the OS when 'aWindow->Show()' used.

// FileFilter_View.cpp
FileFilter_View::FileFilter_View(BRect rect, char *name) // Define 'FileFilter_View' constructor call.
               : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW) { } ;

void FileFilter_View::AttachedToWindow() { // Sets the character of the view when a view ia attached to a window.
     SetFont(be_plain_font);               // Set font outline to bold.
     SetFontSize(18); } ;                  // Set font size.

void FileFilter_View::Draw(BRect updateRect) { /* Draws in view when show is called. */
     X=20.0; Y=30.0; MovePenTo(BPoint(X,Y)); SetHighColor(blackColor); // Set pen to Black.
     DrawString("Drag and drog files to be filtered onto this window.");
     X=20.0; Y=Y+30.0; MovePenTo(BPoint(X,Y));
     DrawString("Otherwise select files to be filtered using the MenuBar.");
     X=20.0; Y=Y+30.0; MovePenTo(BPoint(X,Y));
     DrawString("FileType of filtered files can also be set using Menus."); }

// FileFilter_Window.h
class FileFilter_Window : public BWindow {         // Define the class object FileFilter_Window.
 public:       FileFilter_Window(BRect frame);     // Constructor for a window thread.
  virtual bool QuitRequested();                    // Add function is close box is clicked.
  virtual void MessageReceived(BMessage *message); // Setup to recieve menu & windows messages.
 private:
  BMenuBar     *menubar; } ;                       // Define object to hold all menu structures.

// FileFilter_Window.cpp
FileFilter_Window::FileFilter_Window(BRect frame) // Define 'FileFilter_Window' constructor call.
 : BWindow(frame,                   // Frame area of the window
           "File Filter Window",    // Name of the window, appears in the Tab on top of the window.
           B_TITLED_WINDOW,         // Type of window (Look & Feel)
           B_NOT_RESIZABLE )        // Behavior of the window (resize knob, etc)
{
 BRect rMenu;                       // Set the Window Bpoints into the rectangle object(aRect) to define a MenuBar.
 BMenu *menu;                       // 
 rMenu = Bounds();                  // Find active area of view.
 menubar = new BMenuBar(rMenu,      // Invoke a new dynamically allocated menubar object inside the window
          "Filter menu_bar");       // by calling the constructor. Plus supply a ID name for the menu.
 AddChild(menubar);                 // Add menubar to window created.
 
 menu = new BMenu("File"); // Adds File menu to menu bar.
 menu->AddItem(new BMenuItem("Filter File(s)",new BMessage(MENU_FILE_FILTER)  )); // Filter File.
 menu->AddItem(new BMenuItem("About File Filter",new BMessage(MENU_FILE_ABOUT)  ));  // About the program.
 menu->AddItem(new BMenuItem("Quit",       new BMessage(MENU_FILE_QUIT),'Q')); // Exit program.
 menubar->AddItem(menu); 

 menu = new BMenu("File Types"); // Adds File Type menu to menu bar.
 menu->AddItem(new BMenuItem("No File Type",   new BMessage(MENU_FILETYPE_0))); // No FileType.
 menu->AddItem(new BMenuItem("Text",           new BMessage(MENU_FILETYPE_1))); // Basic Text FileType.
 menu->AddItem(new BMenuItem("Text/Plain",     new BMessage(MENU_FILETYPE_2))); // Plain Text FileType.
 menu->AddItem(new BMenuItem("Text/HTML",      new BMessage(MENU_FILETYPE_3))); // Hyper Text Markup Lanuage FileType.
 menu->AddItem(new BMenuItem("Text/RTF",       new BMessage(MENU_FILETYPE_4))); // Rich Text Format FileType.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("E-mail",         new BMessage(MENU_FILETYPE_5))); // E-mail FileType.
 menu->AddItem(new BMenuItem("Source Code",    new BMessage(MENU_FILETYPE_6))); // Source Code FileType.
 menu->AddSeparatorItem();
 menu->AddItem(new BMenuItem("UseNet Message", new BMessage(MENU_FILETYPE_7))); // UseNet Message FileType.
 menu->AddItem(new BMenuItem("RFC822 Message", new BMessage(MENU_FILETYPE_8))); // Message RFC822 FileType.
 menu->SetRadioMode(true); // Puts check mark beside selected function.
 menubar->AddItem(menu); 
 Show(); } ;

bool FileFilter_Window::QuitRequested() { // Close button on window pressed. A Blooper call to Bwindows. 
   be_app->PostMessage(B_QUIT_REQUESTED); // Send a message to the global variable 'be_app' to tell the program to quit.
   return(true); } ;                      // Tell the system (Blooper) to go ahead and close window.

// FileFilter_Window::MessageReceived.  Called when a message is received by our application.
void FileFilter_Window::MessageReceived(BMessage *message) {
 switch(message->what) {
    case B_SIMPLE_DATA: {
       for (refs=0; message->FindRef("refs", refs, &file_entry_ref) == B_NO_ERROR; refs++) { // Process file if no error finding next file ref.
           File_Error = File_B_Entry.SetTo(&file_entry_ref,false);        // Convert a entry_ref to a BEntry.
           File_Error = File_B_Entry.GetPath(&File_B_Path);               // Get BPath from BEntry.

           File_Error = File_Object.SetTo(&file_entry_ref, B_READ_ONLY);  // Set BFile object to filter.  ReadOnly protects file from changes.
           File_Error = File_Object.GetSize(&File_Size);                  // Get size of file to filter.
           File_Buffer = new uchar[File_Size+255];                        // Create file buffer with extra buffering for overruns.
           Bytes_Read = File_Object.Read(File_Buffer, (size_t)File_Size); // Read file into buffer for filtering.
           File_Object.Unset();                                           // Close File.

           for (Index_Counter = 0; Index_Counter < File_Size; Index_Counter++) { // Process entire buffered file.
               ASC = (int)File_Buffer[Index_Counter] & 127; // Get a single character. Filter out high bit.
               if ((ASC > 31)|(ASC == 13)|(ASC == 10)) {    // Test if printable.
                  File_Buffer[Index_Counter] = (char)ASC;   // Write printable filtered byte into buffer.
               } else {                                     // If not printable then,
                  File_Buffer[Index_Counter] = 64; } }      // Write replacement byte into buffer. Can be any printable character.

           strcpy(name,File_B_Path.Path()); strcat(name,"-Filtered");                     // Rename file path/name from BEntry.
           File_Error = File_Object.SetTo(name, B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE); // Set BFile object to filter.
           File_Error = File_Object.Write((void *)File_Buffer, (size_t)File_Size);        // Write filtered file.
           if ( FileType>0 ) {
              if (FileType == 1) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text", 5); }
              if (FileType == 2) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/plain", 11); }
              if (FileType == 3) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/html", 10); }
              if (FileType == 4) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/rtf", 9); }
              if (FileType == 5) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/x-email", 13); }
              if (FileType == 6) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/x-source-code", 19); }
              if (FileType == 7) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "message/news", 13); }
              if (FileType == 8) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "message/rfc822", 15); } }
           File_Object.Unset(); delete []File_Buffer; } break; } ; // Close File.  Delete array.

  case MENU_FILE_FILTER: { // Filter a file.
       OpenFilePanel = new BFilePanel(B_OPEN_PANEL,NULL,NULL,0,true,NULL,NULL,false,true); // Create the open a file panel.
       OpenFilePanel->Show(); // Display open a file panel, get file name to print.   
       break; } ;             // End of case

  case MENU_FILE_ABOUT: { // Displays alert. About the program.
       be_app->PostMessage(B_ABOUT_REQUESTED); break; } ; // Sends an 'ABOUT' message to BApplication.  

  case MENU_FILE_QUIT: {  // Exit program.
       be_app->PostMessage(B_QUIT_REQUESTED); break; } ; // Sends an 'QUIT' message to BApplication.  

  case MENU_FILETYPE_0: { FileType = 0; break; } ; // No FileType.
  case MENU_FILETYPE_1: { FileType = 1; break; } ; // Basic Text FileType.
  case MENU_FILETYPE_2: { FileType = 2; break; } ; // Plain Text FileType.
  case MENU_FILETYPE_3: { FileType = 3; break; } ; // Hyper Text Markup Lanuage FileType.
  case MENU_FILETYPE_4: { FileType = 4; break; } ; // Rich Text Format FileType.
  case MENU_FILETYPE_5: { FileType = 5; break; } ; // E-mail FileType.
  case MENU_FILETYPE_6: { FileType = 6; break; } ; // Source Code FileType.
  case MENU_FILETYPE_7: { FileType = 7; break; } ; // UseNet Message FileType.
  case MENU_FILETYPE_8: { FileType = 8; break; } ; // Message RFC822 FileType.

  default: {
       BWindow::MessageReceived(message); break; } ; } ; }

// FileFilter_World.h
class FileFilter_Application : public BApplication {  // Define the class object FileFilter_Application.
 public:       FileFilter_Application();              // Constructor for an application thread.
  virtual void AboutRequested();                      // Recieved About message.
  virtual void MessageReceived(BMessage *message);    // ??????????????
  virtual void RefsReceived(BMessage *message);       // Recieve messages from file panel.
  virtual void ArgvReceived(int32 argc, char **argv); //  
} ;

// FileFilter_World.cpp  The main() function's only real job in a basic BeOS application is to create the BApplication object and run it.
int main(int32 argc, char **argv) {
  BString Usage = "Usage:  "; Usage.Append(argv[0]); Usage.Append(" VERSION | HELP | <FileNames> ......"); File_Name = argv[0];

  if ( argc < 2 ) {
     printf("Missing arguement. %s \n",Usage.String()); // CLI program.
     FileFilter_Application myApplication; // Create an Be_app instance?
     myApplication.Run();                  // Run the app thread.  Start to process messages.
     return(0); }                          // Return 0 for no errors.

  if ( argc == 2 ) {
     if ( strcmp("VERSION",argv[1])==0 | strcmp("version",argv[1])==0 | strcmp("-V",argv[1])==0 | strcmp("-v",argv[1])==0) {
        printf(" %s \n",Usage.String()); 
        printf(" Freeware Version 0.70 File Filter program.\n");
        printf(" Written by Earl Colby Pottinger.\n");
        printf(" June 6, 2005.  Whitby, Ontario, Canada.\n\n");
        return(0); } } // Return 0 for no errors.

  if ( argc == 2 ) {
     if (strcmp("HELP",argv[1])==0 | strcmp("help",argv[1])==0 | strcmp("-H",argv[1])==0 | strcmp("-h",argv[1])==0) {
        printf(" GUI and CLI File Filter Version 0.70\n"); 
        printf(" ====================================\n\n");
        printf(" Ways to get Version Info:\n");
        printf(" =========================.\n");
        printf(" %s VERSION\n",File_Name.String());
        printf(" %s version\n",File_Name.String());
        printf(" %s -V\n",File_Name.String());
        printf(" %s -v\n\n",File_Name.String());
        printf(" Ways to get Help Info:\n");
        printf(" ======================\n");
        printf(" %s HELP\n",File_Name.String());
        printf(" %s help\n",File_Name.String());
        printf(" %s -H\n",File_Name.String());
        printf(" %s -h\n\n",File_Name.String());
        printf(" This program filters out out the unprintable characters in a file.  It replaces these characters with a printable '@'.\n");
        printf(" Then the program saves the filtered file into a new file with the same name as the old one but with -Filtered appended\n");
        printf(" to the file name.\n\n");
        printf(" To access the GUI version of this program just double click on the supplied icon.\n\n");
        return(0); } } // Return 0 for no errors.
 
  if ( argc > 1 ) {
     for (n=1; n<argc; n++) {
         File_Name = argv[n]; printf("Filtering unprintable characters from %s \n",File_Name.String());
         File_Error = File_Object.SetTo(File_Name.String(), B_READ_ONLY);        // Set BFile object to filter.  ReadOnly protects file from changes.
         File_Error = File_Object.GetSize(&File_Size);                  // Get size of file to filter.
         File_Buffer = new uchar[File_Size+255];                        // Create file buffer with extra buffering for overruns.
         Bytes_Read = File_Object.Read(File_Buffer, (size_t)File_Size); // Read file into buffer for filtering.
         File_Object.Unset();                                           // Close File.

         for (Index_Counter = 0; Index_Counter < File_Size; Index_Counter++) { // Process entire buffered file.
             ASC = (int)File_Buffer[Index_Counter] & 127; // Get a single character. Filter out high bit.
             if ((ASC > 31)|(ASC == 13)|(ASC == 10)) {    // Test if printable.
                File_Buffer[Index_Counter] = (char)ASC;   // Write printable filtered byte into buffer.
             } else {                                     // If not printable then,
                File_Buffer[Index_Counter] = 64; } }      // Write replacement byte into buffer. Can be any printable character.

         File_Name.Append("-Filtered");// Rename file path/name from BEntry.
         File_Error = File_Object.SetTo(File_Name.String(), B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE); // Set BFile object to filter.
         File_Error = File_Object.Write((void *)File_Buffer, (size_t)File_Size);             // Write filtered file.
         File_Object.Unset(); delete []File_Buffer; } }// Close File.  Delete array.
 } ;

FileFilter_Application::FileFilter_Application() : BApplication(APP_SIGNATURE) { // Define 'FileFilter_Application' constructor call. 
 FileFilter_Window *aWindow;      // Create a pointer(*aWindow) to an object(FileFilter_Window).
 FileFilter_View   *aView;        // Create a pointer(*aView) to an object(FileFilter_View).
 BRect             aRect;         // Set up a rectangle object(aRect). Size of Window to open.

// Create a window. 
 aRect.Set(32, 32, 500, 200);            // Set the Screen Bpoints into the rectangle object(aRect) to define a window.
 aWindow = new FileFilter_Window(aRect); // Invoke a new dynamically allocated window object by calling the constructor.
 aWindow->Show();                        // Make window visible.

// Create a view.
 aRect.Set(0, 19, 499, 199);            // Set the Window Bpoints into the rectangle object(aRect) to define a view.
 aView = new FileFilter_View(aRect,     // Invoke a new dynamically allocated view object inside the window
         "FileFilter__View_ID_Name" ) ; // by calling the constructor. Plus supply a ID name for the view.
 aWindow->AddChild(aView);              // Add view to window.
 aWindow->Show(); } ;                   // Redraw window with the view attached.

// ::AboutRequested. Recieved About message.
void FileFilter_Application::AboutRequested() { // Displays alert with warning.
 alert = new BAlert("About BAlert","About FileFilter 0.70.\n
                                    Removes unprintable characters from a file.\n
                                    Written by Earl Colby Pottinger.\n
                                    June 6, 2005.  Whitby, Ontario, Canada.","OK");
  result = alert->Go(); }

// FileFilter_App::MessageReceived.  Handle incoming messages. 
void FileFilter_Application::MessageReceived(BMessage *message) { // ???????????????
 switch(message->what) {                                        // ???????????????
  default: { BApplication::MessageReceived(message); break; }   // ???????????????
} ; }

// FileFilter_App::RefsReceived.  Handle a refs received message.
// The message specifies where to get it (see BFilePanel in the Storage Kit chapter of the Be Book).
void FileFilter_Application::RefsReceived(BMessage *message) {
  for (refs=0; message->FindRef("refs", refs, &file_entry_ref) == B_NO_ERROR; refs++) { // Process file if no error finding next file ref.
      File_Error = File_B_Entry.SetTo(&file_entry_ref,false); // Convert a entry_ref to a BEntry.
      File_Error = File_B_Entry.GetPath(&File_B_Path);        // Get BPath from BEntry.

      File_Error = File_Object.SetTo(&file_entry_ref, B_READ_ONLY);  // Set BFile object to filter.  ReadOnly protects file from changes.
      File_Error = File_Object.GetSize(&File_Size);                  // Get size of file to filter.
      File_Buffer = new uchar[File_Size+255];                        // Create file buffer with extra buffering for overruns.
      Bytes_Read = File_Object.Read(File_Buffer, (size_t)File_Size); // Read file into buffer for filtering.
      File_Object.Unset();                                           // Close File.

      for (Index_Counter = 0; Index_Counter < File_Size; Index_Counter++) { // Process entire buffered file.
          ASC = (int)File_Buffer[Index_Counter] & 127; // Get a single character. Filter out high bit.
          if ((ASC > 31)|(ASC == 13)|(ASC == 10)) {    // Test if printable.
             File_Buffer[Index_Counter] = (char)ASC;   // Write printable filtered byte into buffer.
          } else {                                     // If not printable then,
             File_Buffer[Index_Counter] = 64; } }      // Write replacement byte into buffer. Can be any printable character.

      strcpy(name,File_B_Path.Path()); strcat(name,"-Filtered");// Rename file path/name from BEntry.
      File_Error = File_Object.SetTo(name, B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE); // Set BFile object to filter.
      File_Error = File_Object.Write((void *)File_Buffer, (size_t)File_Size);             // Write filtered file.
      if ( FileType>0 ) {
         if (FileType == 1) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text", 5); }
         if (FileType == 2) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/plain", 11); }
         if (FileType == 3) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/html", 10); }
         if (FileType == 4) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/rtf", 9); }
         if (FileType == 5) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/x-email", 13); }
         if (FileType == 6) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "text/x-source-code", 19); }
         if (FileType == 7) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "message/news", 13); }
         if (FileType == 8) { File_Object.WriteAttr("BEOS:TYPE",  B_MIME_TYPE, 0, "message/rfc822", 15); } }
      File_Object.Unset(); delete []File_Buffer; } } // Close File.  Delete array.

// FileFilter_App::ArgvReceived().  CLI argruements recieved.
// Command line arguments are used in launching the app from the shell.
void FileFilter_Application::ArgvReceived(int32 argc, char **argv) {
  if ( argc == 0 ) { alert = new BAlert("BAlert","Argc = 0.","OK"); result = alert->Go(); }
  if ( argc == 1 ) { alert = new BAlert("BAlert","Argc = 1.","OK"); result = alert->Go(); }
  if ( argc == 2 ) { alert = new BAlert("BAlert","Argc = 2.","OK"); result = alert->Go(); }
  if ( argc == 3 ) { alert = new BAlert("BAlert","Argc = 3.","OK"); result = alert->Go(); }
  if ( argc > 3 )  { alert = new BAlert("BAlert","Argc > 3.","OK"); result = alert->Go(); }
}